"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EventEmitter = require("events");
const kafka_node_1 = require("kafka-node");
const Nats = require("nats");
const util_1 = require("util");
const logs_1 = require("./logs");
const util_2 = require("./util");
/**
 *  Clase para las conexiones y publicaciones sobre
 *  la cola de mensajes.
 */
class Brokers extends util_2.default {
    /**
     * Contructor de la clase.
     * @param kafkaHost Servidores Kafka
     * @param Topic Topico de escucha
     */
    constructor(Conf) {
        //  Contructor padre
        super();
        /**
         * Manejador de eventos
         * @protected
         */
        this.event = new EventEmitter();
        /**
         * Manejador de logs
         * @protected
         */
        this.logs = logs_1.default;
        /**
         * Servidores de kafka
         * @protected
         */
        this.kafkaHost = [];
        /**
         * Hosts de los brokers
         * @protected
         */
        this.Hosts = [];
        // Asignar la configuracion
        this.conf = Conf;
        this.conf.ClearQueque = util_1.isUndefined(Conf.ClearQueque) ? true : Conf.ClearQueque;
        /**
         * Definir topico con soporte para servicios con nomemclatura
         * legada.
         */
        // Ver si es legacy
        if (this.conf.Legacy) {
            // Es un servicio legado
            this.Topic = this.conf.Name;
        }
        else {
            // Crear la combinacion
            this.Topic = `${this.conf.App}_${this.conf.Version}_${this.conf.Name}`;
        }
        // Ver si corresponde a una conexion sobre NATS
        if (process.env.NATS) {
            this.connecNats();
        }
        else {
            this.connectKafka();
        }
    }
    /**
     * Metodo para realizar el envio de datos a la cola de mensajes
     * @param data Datos a enviar
     * @param topic Topico de publicacion
     * @param key Clave unica
     */
    send(data, topic, key = util_2.default.key()) {
        if (process.env.NATS) {
            // Responder la peticion
            this.nc.publish(topic, data);
        }
        else {
            this.producer.send([{
                    key,
                    messages: [data],
                    topic,
                }], (err) => {
                if (err) {
                    // Imprimir el error
                    this.logs.error(err);
                }
            });
        }
    }
    /**
     * Metodo para realizar la conexion Nats
     * @private
     */
    connecNats() {
        // Verificar si se paso la cadena de conexion a kafka
        if (this.conf.Nats) {
            // Asignar la pasada como configuracion
            this.Hosts = this.conf.Nats;
        }
        else {
            // Verificar que se asigno una por variable de ambiente
            if (process.env.NATS) {
                // Recorrer todos los elementos para darle formato
                for (const i of process.env.NATS.split(',')) {
                    this.Hosts.push(`nats://${i}`); // Dar el formato correcto
                }
            }
            else {
                this.logs.error('No se proporciono la configuracion de NATS');
                process.exit(-1); // Finalizar el programa
            }
        }
        // Realizar la conexion a NATS - nc.currentServer.url.host
        this.nc = Nats.connect({ servers: this.Hosts, name: this.Topic });
        // Evento que indica la conexion
        this.nc.on('connect', (nc) => {
            const text = `Escuchando el topico ${this.Topic} en Nats ${this.nc.currentServer.url.host}`;
            // Notificar que la conexion fue exitosa.
            this.logs.info(text);
            // Realizar la conexion de subcripcion al topico
            nc.subscribe(this.Topic, { queue: this.Topic }, (msg, reply, subject) => {
                try {
                    // Convertir el mensaje
                    let data = JSON.parse(Buffer.from(msg, 'base64').toString('utf8'));
                    // agregar el ID para el reply
                    if (!data.uuid) {
                        data = Object.assign(data, { uuid: reply });
                    }
                    // Pasar el mensaje al que lo procesara
                    this.message(data);
                }
                catch (error) {
                    // Disparamos la excepcion
                    this.exception(this.Topic, msg, error);
                }
            });
        });
    }
    /**
     * Metodo para realizar la conexion al broker de kafka
     * @private
     */
    connectKafka() {
        // Verificar si se paso la cadena de conexion a kafka
        if (this.conf.Kafka) {
            // Asignar la pasada como configuracion
            this.kafkaHost = this.conf.Kafka;
        }
        else {
            // Verificar que se asigno una por variable de ambiente
            if (process.env.KAFKA) {
                // Asignar
                this.kafkaHost = process.env.KAFKA.split(',');
            }
            else {
                this.logs.error('No se proporciono la configuracion de KAFKA');
                process.exit(-1); // Finalizar el programa
            }
        }
        // Crear el cliente de kafka.
        this.client = new kafka_node_1.KafkaClient({ kafkaHost: this.kafkaHost.join(',') });
        // Registrar el producer.
        this.producer = new kafka_node_1.Producer(this.client);
        // Eventos del producer
        this.producer.on('ready', () => {
            // Opciones del consumidor
            const options = {
                groupId: this.Topic,
                kafkaHost: this.kafkaHost.join(','),
            };
            // Crear el consumidor
            this.consumer = new kafka_node_1.ConsumerGroup(options, [this.Topic]);
            // Escuchar cuando se realice la conexion
            this.consumer.on('connect', () => {
                const text = `Escuchando el topico ${this.Topic} en el Kafka ${this.kafkaHost.join(',')}`;
                // Notificar que la conexion fue exitosa.
                this.logs.info(text);
            });
            // Llegada de mensajes
            this.consumer.on('message', (message) => {
                try {
                    // Convertir el mensaje
                    const data = JSON.parse(Buffer.from(message.value.toString(), 'base64').toString('utf8'));
                    // Pasar el mensaje al que lo procesara
                    this.message(data);
                }
                catch (error) {
                    // Disparamos la excepcion
                    this.exception(this.Topic, message.value.toString(), error);
                }
            });
            // Errores
            this.consumer.on('error', (err) => {
                // Enviar el error
                this.logs.error(err);
            });
        });
        this.producer.on('error', (err) => {
            // Enviar el error
            this.logs.error(err);
        });
    }
}
exports.default = Brokers;
